<?php

namespace Drupal\activitypub\Controller;

use Drupal\activitypub\Entity\ActivityPubActorInterface;
use Drupal\activitypub\Services\ActivityPubProcessClientInterface;
use Drupal\activitypub\Services\ActivityPubUtilityInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Pager\PagerManagerInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class BaseController extends ControllerBase {

  /**
   * The ActivityPub Utility service.
   *
   * @var \Drupal\activitypub\Services\ActivityPubUtilityInterface
   */
  protected $activityPubUtility;

  /**
   * The path matcher service.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * The ActivityPub process client service.
   *
   * @var \Drupal\activitypub\Services\ActivityPubProcessClientInterface
   */
  private $activityPubProcessClient;

  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerManager;

  /**
   * BaseController constructor
   *
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   The path matcher
   * @param \Drupal\activitypub\Services\ActivityPubProcessClientInterface $activitypub_process_client
   *   The ActivityPub process client
   * @param \Drupal\activitypub\Services\ActivityPubUtilityInterface $activitypub_utility
   *   The ActivityPub utility
   * @param \Drupal\Core\Pager\PagerManagerInterface $pager_manager
   *   The pager manager
   */
  public function __construct(PathMatcherInterface $path_matcher, ActivityPubProcessClientInterface $activitypub_process_client, ActivityPubUtilityInterface $activitypub_utility, PagerManagerInterface $pager_manager) {
    $this->activityPubUtility = $activitypub_utility;
    $this->pathMatcher = $path_matcher;
    $this->activityPubProcessClient = $activitypub_process_client;
    $this->pagerManager = $pager_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('path.matcher'),
      $container->get('activitypub.process.client'),
      $container->get('activitypub.utility'),
      $container->get('pager.manager')
    );
  }

  /**
   * Check that the actor points to the user.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @param \Drupal\activitypub\Entity\ActivityPubActorInterface $activitypub_actor
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function actorUserCheck(UserInterface $user, ActivityPubActorInterface $activitypub_actor) {

    if ($user->id() == $activitypub_actor->getTargetEntityId()) {
      return AccessResult::allowed();
    }

    return AccessResult::forbidden();
  }

}
