<?php

namespace Drupal\activitypub\Controller;

use Drupal\activitypub\Entity\ActivityPubActivityInterface;
use Drupal\comment\CommentInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\node\NodeInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

class EntityController extends BaseController {

  /**
   * Node self routing callback with activity+json accept header.
   *
   * @param \Drupal\node\NodeInterface $node
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse|\Symfony\Component\HttpFoundation\JsonResponse
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function node(NodeInterface $node) {
    return $this->json($node);
  }

  /**
   * Comment self routing callback with activity+json accept header.
   *
   * @param \Drupal\comment\CommentInterface $comment
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse|\Symfony\Component\HttpFoundation\JsonResponse
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function comment(CommentInterface $comment) {
    return $this->json($comment);
  }

  /**
   * Node self routing callback with activity+json accept header.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return \Drupal\Core\Cache\CacheableJsonResponse|\Symfony\Component\HttpFoundation\JsonResponse
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function json(EntityInterface $entity) {
    /** @var \Drupal\activitypub\Entity\Storage\ActivityPubActorStorageInterface $actorStorage */
    $actorStorage = $this->entityTypeManager()->getStorage('activitypub_actor');
    $actor = $actorStorage->loadActorByEntityIdAndType($entity->getOwnerId(), 'person');

    /** @var \Drupal\activitypub\Entity\Storage\ActivityPubActivityStorageInterface $activityStorage */
    /** @var \Drupal\activitypub\Entity\ActivityPubActivityInterface $activity */
    $activity = NULL;
    $activityStorage = $this->entityTypeManager()->getStorage('activitypub_activity');
    $activities = $activityStorage->loadByProperties(['entity_type_id' => $entity->getEntityTypeId(), 'entity_id' => $entity->id(), 'collection' => 'outbox']);
    if (!empty($activities)) {
      $activity = array_shift($activities);
    }

    if ($actor && $activity) {
      $build = $activity->buildActivity();
      if (is_array($build['object'])) {
        $output = $build['object'];
      }
      else {
        $output = $build;
      }
      $output['@context'] = ActivityPubActivityInterface::CONTEXT_URL;
      $response = new JsonResponse($output, 200);
      $response->headers->set('Content-Type', 'application/activity+json');
      return $response;
    }
    else {
      return new JsonResponse(NULL, 404);
    }
  }

}
