<?php

namespace Drupal\activitypub\Entity\Storage;

use Drupal\Core\Entity\Sql\SqlContentEntityStorage;

/**
 * Storage class for ActivityPub activities.
 */
class ActivityPubActivityStorage extends SqlContentEntityStorage implements ActivityPubActivityStorageInterface {

  /**
   * {@inheritdoc}
   */
  public function getActivities($conditions = []) {
    $query = $this->getBaseQuery($conditions);
    $ids = $query->execute()->fetchAllKeyed(0, 0);
    return $this->loadMultiple($ids);
  }

  /**
   * {@inheritdoc}
   */
  public function getActivityCount($conditions = []) {
    $query = $this->getBaseQuery($conditions);
    return $query->countQuery()->execute()->fetchField();
  }

  /**
   * Return a select query.
   *
   * @param array $conditions
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   */
  protected function getBaseQuery($conditions = []) {
    $query = $this->database->select($this->getBaseTable(), 't');
    $query->fields('t', ['id']);
    foreach ($conditions as $key => $value) {
      $operator = is_array($value) ? 'IN' : '=';
      if (strpos($key, '!') !== FALSE) {
        $operator = is_array($value) ? 'NOT IN' : '<>';
      }
      $query->condition($key, $value, $operator);
    }
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function changeReadStatus($status) {
    $is_read_condition = $status ? 0 : 1;
    $query = $this->database->update('activitypub_activity')
      ->fields(['is_read' => $status])
      ->condition('type', ['Create', 'Like', 'Announce'], 'IN')
      ->condition('is_read', $is_read_condition);
    $query->execute();
  }

}
