<?php

namespace Drupal\activitypub\Form;

use Drupal\activitypub\Entity\ActivityPubActorInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\user\UserInterface;

class ConfirmActivityPubActorDeleteForm extends ConfirmFormBase {

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete the Actor?');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return Url::fromRoute('activitypub.user.settings', ['user' => $this->currentUser()->id()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'activitypub_actor_delete_form';
  }

  /**
   * Check that the actor points to the user.
   *
   * @param \Drupal\user\UserInterface $user
   * @param \Drupal\activitypub\Entity\ActivityPubActorInterface $activitypub_actor
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function actorUserCheck(UserInterface $user, ActivityPubActorInterface $activitypub_actor) {

    if ($user->id() == $activitypub_actor->getTargetEntityId()) {
      return AccessResult::allowed();
    }

    return AccessResult::forbidden();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, UserInterface $user = NULL, ActivityPubActorInterface $activitypub_actor = NULL) {
    $form = parent::buildForm($form, $form_state);

    $form['activitypub_actor'] = [
      '#type' => 'value',
      '#value' => $activitypub_actor,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\activitypub\Entity\ActivityPubActorInterface $activitypub_actor */
    $activitypub_actor = $form_state->getValue('activitypub_actor');
    $activitypub_actor->delete();
    $this->messenger()->addMessage($this->t('ActivityPub profile has been deleted.'));
    Cache::invalidateTags(['user:' . $activitypub_actor->getOwnerId()]);
    $form_state->setRedirectUrl(Url::fromRoute('activitypub.user.settings', ['user' => $activitypub_actor->getOwnerId()]));
  }

}
